import os
import sys
import inspect
import logging

from pytest_unordered import unordered

THIS_DIR = os.path.abspath(os.path.dirname(__file__))
RES_DIR = os.path.abspath(
    os.path.join(THIS_DIR, '../res')
)
sys.path.append(RES_DIR)

import bunch_of_imports
from fast_dep import GraphBuilder

def test_bunch():
    builder = GraphBuilder()
    print(inspect.getsource(bunch_of_imports))
    graph = builder.build(
        inspect.getsource(bunch_of_imports),
        package=''
    )

    # Test caching
    graph = builder.build(
        inspect.getsource(bunch_of_imports),
        package=''
    )


IMPORTS = [
    {
        'source' :"""
        from os import (
            path, # Lolz
        # Yo

                # Yo
            getcwd    ,
            getenv,
        )
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'os',
            'names': [
                { 'name': 'path' },
                { 'name': 'getcwd' },
                { 'name': 'getenv' },
            ]
        }]
    },
    {
        'source' :"""
        import \
            os
        """,
        # ----------------
        'expect' : [{
            'type': 'import',
            'names': [
                { 'name': 'os' },
            ]
        }]
    },
    {
        'source' :"""
        from\
            pathlib\
            import\
                Path
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'pathlib',
            'names': [
                { 'name': 'Path' },
            ]
        }]
    },
    {
        'source' :"""
        from path\
            import\
                lib
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'path',
            'names': [
                { 'name': 'lib' },
            ]
        }]
    },
]


IMPORTS = [
    {
        'source' :"""
        import _private_module
        """,
        # ----------------
        'expect' : [{
            'type': 'import',
            'names': [
                { 'name': '_private_module' },
            ]
        }]
    },
    {
        'source' :"""
        from _private_module import _private_name
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': '_private_module',
            'names': [
                { 'name': '_private_name' },
            ]
        }]
    },
    {
        'source' :"""
        import      os
        """,
        # ----------------
        'expect' : [{
            'type': 'import',
            'names': [
                { 'name': 'os' },
            ]
        }]
    },
    {
        'source' :"""
        from os import (
            path,
            getcwd,
        )
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'os',
            'names': [
                { 'name': 'path' },
                { 'name': 'getcwd' },
            ]
        }]
    },
    {
        'source' :"""
        from os import (
            path,
            getcwd
        )
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'os',
            'names': [
                { 'name': 'path' },
                { 'name': 'getcwd' },
            ]
        }]
    },
    {
        'source' :"""
        import os, \
            sys
        """,
        # ----------------
        'expect' : [{
            'type': 'import',
            'names': [
                { 'name': 'os' },
                { 'name': 'sys' },
            ]
        }]
    },
    {
        'source' :"""
        from os import (
            path,
            getcwd,
            getenv,)
        """,
        # ----------------
        'expect' : [{
            'type': 'import_from',
            'level': 0,
            'module': 'os',
            'names': [
                { 'name': 'path' },
                { 'name': 'getcwd' },
                { 'name': 'getenv' },
            ]
        }]
    },
]


import os
import sys
import inspect
import logging

from pytest_unordered import unordered

THIS_DIR = os.path.abspath(os.path.dirname(__file__))
RES_DIR = os.path.abspath(
    os.path.join(THIS_DIR, '../res')
)
sys.path.append(RES_DIR)

from test_packages.imports import *
from fast_dep import GraphBuilder

def test_module_dep():
    builder = GraphBuilder()
    print(inspect.getsource(import_module_dep))
    graph = builder.build(
        inspect.getsource(import_module_dep),
        package='test_packages'
    )

    assert graph.size() == 4

    expected_nodes = [
        {
            'name': 'test_packages.module_dep.file',
            'dependencies': 0,
            'dependents': {'test_packages.module_dep'},
            'depth': 2
        },
        {
            'name': 'test_packages.module_dep',
            'dependencies': 1,
            'dependents': {'<terminal>'},
            'depth': 1
        },
        {
            'name': 'test_packages',
            'dependencies': 0,
            'dependents': {'<terminal>'},
            'depth': 1
        },
        {
            'name': '<terminal>',
            'dependencies': 2,
            'dependents': set(),
            'depth': 0
        }
    ]

    for expected in expected_nodes:
        actual = graph.get(expected['name'])
        print(expected['name'])
        assert actual.dependencies == expected['dependencies']
        assert actual.dependents == expected['dependents']
        assert actual.depth == expected['depth']

    scope_test = graph.get_all_scoped('test_packages.module_dep')
    assert [node.name for node in scope_test] == unordered([
        'test_packages.module_dep.file',
        'test_packages.module_dep',
    ])

    graph = builder.build(
        inspect.getsource(import_module_dep),
        package='test_packages'
    )




# Hacky until the following gets patched
# - https://github.com/PyO3/pyo3/issues/759
import fast_dep
parser = fast_dep.parser

from res.import_easy import IMPORTS as IMPORTS_EASY
from res.import_hard import IMPORTS as IMPORTS_HARD
from res.import_wtf import IMPORTS as IMPORTS_WTF
from res.not_imports import NOT_IMPORTS

def test_easy():
    for variant in IMPORTS_EASY:
        parsed = parser.parse(variant['source'])

        assert len(parsed) == len(variant['expect'])
        assert parsed == variant['expect']

def test_hard():
    for variant in IMPORTS_HARD:
        parsed = parser.parse(variant['source'])

        assert len(parsed) == len(variant['expect'])
        assert parsed == variant['expect']

def test_wtf():
    for variant in IMPORTS_WTF:
        parsed = parser.parse(variant['source'])

        assert len(parsed) == len(variant['expect'])
        assert parsed == variant['expect']

def test_wtf():
    for variant in NOT_IMPORTS:
        parsed = parser.parse(variant['source'])

        assert len(parsed) == len(variant['expect'])
        assert parsed == variant['expect']


# Simple
# ---------------
import os
import os,sys
import os, sys
import os as a, sys as b

from os import path
from os import path, getcwd
from os import path as p, getcwd as whereami

'import file'
'from blah import z'

# Harder
# ----------------
import _private_module
from _private_module import _private_name

from . import blah

import      os
from        os     import path
"""
import os
from os import path
"""

from os import (
    path,
    getcwd,
)

from os import (
    path,
    getcwd
)

import os, \
    sys

# Wtf
# ----------------

from os import (
    path,
    getcwd,
    getenv,)

from os import (
    path, # Lolz
# Yo

        # Yo
    getcwd    ,
    getenv,
)

from os import (
    path,
    getcwd,
    getenv,)

import \
    os

from\
    pathlib\
    import\
        Path

from path\
    import\
        lib


NOT_IMPORTS = [
    {
        'source' :'''
        """
        import os
        from os import path
        """
        ''',
        # ----------------
        'expect' : []
    },
    {
        'source' :"""
        '''
        import os
        from os import path
        '''
        """,
        # ----------------
        'expect' : []
    },
    # Honestly need to think about if I need to detect if the below are valid python or not too.
    {
        'source' :"""
        import (os, sys)
        """,
        # ----------------
        'expect' : []
    },
    {
        'source' :"""
        importos
        """,
        # ----------------
        'expect' : []
    },
    {
        'source' :"""
        fromosimportpath
        """,
        # ----------------
        'expect' : []
    },
    # Currently failing need to patch grammar
    # {
    #     'source' :"""
    #     import os,
    #         sys
    #     """,
    #     # ----------------
    #     'expect' : []
    # },
    {
        'source' :"""
        from os import (
            
        )
        """,
        # ----------------
        'expect' : []
    },
]